@@ -5,9 +5,12 @@ use 5.005;
 use strict;
 use vars qw($VERSION $BASE_EXC_CLASS %CLASSES);
 
+use Scalar::Util qw(blessed);
+
+
 BEGIN { $BASE_EXC_CLASS ||= 'Exception::Class::Base'; }
 
-$VERSION = '1.18';
+$VERSION = '1.21';
 
 sub import
 {
@@ -171,6 +174,16 @@ EOPERL
     $CLASSES{$subclass} = 1;
 }
 
+sub caught
+{
+    my $e = $@;
+
+    return unless blessed($e) && $e->isa( $_[1] );
+    return $e;
+}
+
+sub Classes { sort keys %Exception::Class::CLASSES }
+
 package Exception::Class::Base;
 
 use Class::Data::Inheritable;
@@ -218,7 +231,7 @@ BEGIN
 
 1;
 
-sub Classes { sort keys %Exception::Class::CLASSES }
+sub Classes { Exception::Class::Classes() }
 
 sub throw
 {
@@ -267,9 +280,22 @@ sub _initialize
     $self->{gid}  = $(;
     $self->{egid} = $);
 
+    my @ignore_class   = (__PACKAGE__);
+    my @ignore_package = 'Exception::Class';
+
+    if ( my $i = delete $p{ignore_class} )
+    {
+        push @ignore_class, ( ref($i) eq 'ARRAY' ? @$i : $i );
+    }
+
+    if ( my $i = delete $p{ignore_package} )
+    {
+        push @ignore_package, ( ref($i) eq 'ARRAY' ? @$i : $i );
+    }
+
     $self->{trace} =
-        Devel::StackTrace->new( ignore_class     => __PACKAGE__,
-                                ignore_package   => 'Exception::Class',
+        Devel::StackTrace->new( ignore_class     => \@ignore_class,
+                                ignore_package   => \@ignore_package,
                                 no_refs          => $self->NoRefs,
                                 respect_overload => $self->RespectOverload,
                               );
@@ -505,17 +531,46 @@ hierarchies and will die if it finds one.  It also detects missing
 links in a chain, for example if you declare Bar to be a subclass of
 Foo and never declare Foo.
 
+=head1 Catching Exceptions
+
+C<Exception::Class> provides some syntactic sugar for catching
+exceptions in a safe manner:
+
+ eval { ... }
+
+ if ( my $e = Exception::Class->caught('My::Error') )
+ {
+     cleanup();
+     do_something_with_exception($e);
+ }
+
+The C<caught()> method returns an exception object if the last thrown
+exception is of the given class, or a subclass of that class.
+Otherwise it returns false.
+
+You should B<always> make a copy of the exception object, rather than
+using C<$@> directly.  This is necessary because if your C<cleanup()>
+function uses C<eval>, or calls something which uses it, then C<$@> is
+overwritten.  Copying the exception preserves it for the call to
+C<do_something_with_exception()>.
+
+=head2 Uncatchable Exceptions
+
+Internally, the C<caught()> method will call C<isa()> on the exception
+object.  You could make an exception "uncatchable" by overriding
+C<isa()> in that class like this:
+
+ package Exception::Uncatchable;
+
+ sub isa { shift->rethrow }
+
+Of course, this only works if you always call 
+C<< Exception::Class->caught() >> after an C<eval>.
+
 =head1 Exception::Class::Base CLASS METHODS
 
 =over 4
 
-=item * Classes
-
-Returns a list of the classes that have been defined as subclasses of
-Exception::Class.  Note that this is I<all> the subclasses that have
-been created, so it may include subclasses created by things like CPAN
-modules, etc.
-
 =item * Trace($boolean)
 
 Each C<Exception::Class::Base> subclass can be set individually to
@@ -571,15 +626,19 @@ an array.
 
 =item * throw( error => $error )
 
-This method creates a new C<Exception::Class::Base> object with the
-given error message.  If no error message is given, C<$!> is used.  It
-then die's with this object as its argument.
+This method creates a new object with the given error message.  If no
+error message is given, C<$!> is used.  It then die's with this object
+as its argument.
 
 This method also takes a C<show_trace> parameter which indicates
 whether or not the particular exception object being created should
 show a stacktrace when its C<as_string()> method is called.  This
 overrides the value of C<Trace()> for this class if it is given.
 
+The frames included in the trace can be controlled by the C<ignore_class>
+and C<ignore_package> parameters. These are passed directly to
+Devel::Stacktrace's constructor. See C<Devel::Stacktrace> for more details.
+
 If only a single value is given to the constructor it is assumed to be
 the message parameter.
 
@@ -691,7 +750,7 @@ overridden by a subclass.  See below for details.
 The C<Exception::Class::Base> object is overloaded so that
 stringification produces a normal error message.  It just calls the
 as_string method described above.  This means that you can just
-C<print $@> after an eval and not worry about whether or not its an
+C<print $@> after an C<eval> and not worry about whether or not its an
 actual object.  It also means an application or module could do this:
 
  $SIG{__DIE__} = sub { Exception::Class::Base->throw( error => join '', @_ ); };
@@ -765,6 +824,17 @@ C<Exception::Class::Base>.  You should feel free to subclass any of
 the methods documented above.  For example, you may want to subclass
 C<new()> to add additional information to your exception objects.
 
+=head1 Exception::Class FUNCTIONS
+
+The C<Exception::Class> method offers one function, C<Classes()>,
+which is not exported.  This method returns a list of the classes that
+have been created by calling the C<Exception::Class> import() method.
+Note that this is I<all> the subclasses that have been created, so it
+may include subclasses created by things like CPAN modules, etc.  Also
+note that if you simply define a subclass via the normal Perl method
+of setting C<@ISA> or C<use base>, then your subclass will not be
+included.
+
 =head1 OTHER EXCEPTION MODULES (try/catch syntax)
 
 If you are interested in adding try/catch/finally syntactic sugar to
@@ -776,7 +846,8 @@ If you would prefer to use the C<Exception::Class::Base> class
 included with this module, you'll have to add this to your code
 somewhere:
 
-  push @Exception::Class::Base::ISA, 'Error';
+  push @Exception::Class::Base::ISA, 'Error'
+      unless Exception::Class::Base->isa('Error');
 
 It's a hack but apparently it works.
 
